// coding: utf-8
/** Created by: Udo Schmal | https://www.gocher.me/ */
(function () {
  'use strict';
  function youtube(el) {
    var player, id;
    // don't send referrer infos to external services
    if (!document.querySelector('meta[name="referrer"]')) {
      let meta = document.createElement('meta');
      meta.name = 'referrer';
      meta.content = 'no-referrer';
      document.head.appendChild(meta);
    }
    // get YouTube video id
    if (el.dataset.src && el.dataset.src.length === 11) {
      id = el.dataset.src;
    } else {
      let src = el.dataset.src;
      let regExp = /^.*((youtube\/)|(v\/)|(\/u\/\w\/)|(embed\/)|(watch\?))\??v?=?([^#&?]*).*/;
      let match = src.match(regExp);
      id = (match && match[7].length == 11) ? match[7] : src;
    }
    el.id = id;
    // show in popup, other defined target or replace el with player
    var target = el.dataset.target || id;

    // create placeholder background image
    if (el.dataset.poster) {
      el.style.backgroundImage = "url('" + el.dataset.poster + "')";
    } else {
      // high quality: https://img.youtube.com/vi/<insert-youtube-video-id-here>/hqdefault.jpg
      // medium quality: https://img.youtube.com/vi/<insert-youtube-video-id-here>/mqdefault.jpg
      // standard quality: https://img.youtube.com/vi/<insert-youtube-video-id-here>/sddefault.jpg
      // maximum resolution: https://img.youtube.com/vi/<insert-youtube-video-id-here>/maxresdefault.jpg
      //el.style.backgroundImage = "url('https://i.ytimg.com/vi_webp/" + id + "/sddefault.webp')"; // not supported by IE11
      el.style.backgroundImage = "url('https://img.youtube.com/vi/" + id + "/sddefault.jpg')";
    }
    if (el.dataset.width && el.dataset.height) {
      el.style.width = el.dataset.width + 'px';
      el.style.height = el.dataset.height + 'px';
      el.style.paddingTop = '0';
    }
    // show placeholder title
    if (el.title) {
      let title = document.createElement('div');
      title.className = 'title';
      title.appendChild(document.createTextNode(el.title));
      el.appendChild(title);
    }
    // show placeholder external message
    if (el.dataset.external) {
      let external = document.createElement('div');
      external.className = 'external';
      let language = window.navigator.userLanguage || window.navigator.language;
      if (!language) {
        language = 'en';
      }
      let txt;
      switch (language.substr(0,2)) {
        case 'de': txt = 'Läd externe Inhalte von YouTube'; break;
        case 'fr': txt = 'Charger du contenu externe à partir de YouTube'; break;
        case 'it': txt = 'Carica contenuti esterni da YouTube'; break;
        case 'es': txt = 'Cargar contenido externo de YouTube'; break;
        default: txt = 'Load external Content from YouTube'; break;
      }
      external.appendChild(document.createTextNode(txt));
      el.appendChild(external);
    }
    // show placeholder youtube like play button
    var play = document.createElement('div');
    play.className = 'play';
    el.appendChild(play);

    // create popup
    function createPopup() {
      var overlay = document.createElement('div');
      overlay.id = 'overlay';
      document.body.appendChild(overlay);
      var content = document.createElement('div');
      content.id = 'popup-content';
      overlay.appendChild(content);
      var close = document.createElement('span');
      close.className = 'popup-close';
      overlay.appendChild(close);
      close.addEventListener('click', function (event) {
        player.stopVideo();
        document.body.removeChild(overlay);
      });
      return 'popup-content';
    }
    // youtube player API
    function onPlayerReady(event) {
      event.target.playVideo();
    }
    function onPlayerStateChange(event) {
      if (event.data == window.YT.PlayerState.ENDED) {
        player.stopVideo();
        document.body.removeChild(overlay);
      }
    }
    // create youtube player
    function createPlayer() {
      window.YT.ready(function() {
        var dest, obj = {
          videoId: id,
          host: 'https://www.youtube-nocookie.com',
          events: {
            'onReady': onPlayerReady
          }
        };
        var sizes = [
          { width: '426', height: '240' },
          { width: '640', height: '360' },
          { width: '854', height: '480' },
          { width: '1280', height: '720' },
          { width: '1920', height: '1080' }
        ];
        if (target == 'popup') {
          dest = createPopup();
          obj.events['onStateChange'] = onPlayerStateChange;
          for(var i = 0; i < sizes.length; i++) {
            if (sizes[i].width <= window.innerWidth) {
              obj.height = sizes[i].height;
              obj.width = sizes[i].width;
            } else {
              break;
            }
          }
        } else {
          dest = target;
          for(var i = 0; i < sizes.length; i++) {
            if (sizes[i].width >= el.offsetWidth) {
              obj.height = sizes[i].height;
              obj.width = sizes[i].width;
              break;
            }
          }
        }
        // create YouTube player
        player = new window.YT.Player(dest, obj);
      });
    }
    // load youtube external content
    function load() {
      // init once
      if (!document.getElementById('YouTubeApi')) {
        // load YouTube API
        var script = document.createElement('script');
        script.id = 'YouTubeApi';
        script.src = 'https://www.youtube.com/iframe_api';
        document.head.appendChild(script);
        script.addEventListener('load', createPlayer);
      } else {
        createPlayer();
      }
      if (target !== 'popup') {
        // remove placeholder click event
        play.removeEventListener('click', load);
      }
    }
    play.addEventListener('click', load);
  }

  // Viewport prototype to trigger event if element moves into viewport the fist time
  function Viewport(els, callback) {
    this.els = els;
    this.callback = callback;
    this.active = false;
    var self = this;
    var hidden = [];
    function alreadyObserved(el) {
      for(var i=0; i<hidden.length; i++) {
        if (el === hidden[i]) {
          return true;
        }
      }
      return false;
    }
    for (var i=0; i < els.length; i++) {
      var el = els[i];
      // parent must have position attribute for getBoundingClientRect
      let properties = window.getComputedStyle(el.parentNode);
      if (!properties.getPropertyValue('position')) {
        el.parentNode.style.position = 'relative';
        while(el && el.tagName.toLowerCase() !== 'body') {
          if (window.getComputedStyle(el).display === "none") {
            if (!alreadyObserved(el)) {
              hidden.push(el);
              var observer = new MutationObserver(function(mutations) {
                if (mutations[0].target.style.display !== 'none') {
                  self.handleEvent();
                }
              });
              observer.observe(el, { attributes: true });
              break;
            }
          }
          el = el.parentNode;
        }
      }
    }
    this.handleEvent.bind(this);
    // is already in viewport after dom ready
    function ready(f){
      /complete|loaded/i.test(document.readyState) ? f() : setTimeout(function(){ready(f);},9);
    }
    ready(function(){self.handleEvent();});
    // add event listener to scroll event to check visibility change
    document.addEventListener('scroll', this, true);
    window.addEventListener('resize', this, true);
    window.addEventListener('orientationchange', this, true);

  }

  Viewport.prototype = {
    // check if element is visible
    isVisible: function (el) {
      var style = window.getComputedStyle(el);
      return !!(el.offsetWidth || el.offsetHeight || el.getClientRects().length);
    },
    // check if element is in viewport
    isInViewport: function (el) {
      var bounding = el.getBoundingClientRect();
      return (
        bounding.bottom >= 0 &&
        bounding.right >= 0 &&
        bounding.top <= (window.innerHeight || document.documentElement.clientHeight) &&
        bounding.left <= (window.innerWidth || document.documentElement.clientWidth)
      );
    },
    // handle the visibility check and the resulting action
    handleEvent: function () {
      if (this.active === false) {
        this.active = true;
        for (var i=0; i < this.els.length; i++) {
          if (this.isInViewport(this.els[i]) && this.isVisible(this.els[i])) {
            this.callback(this.els[i]);
          }
        }
        this.active = false;
      }
    }
  };

  // activate viewport for youtube loading
  var media = document.querySelectorAll('.youtube[data-src]');
  if (media.length > 0) {
    if ("IntersectionObserver" in window) {
      // create intersection observer
      let lazyobserver = new IntersectionObserver(function(entries, observer) {
        for (var i=0; i < entries.length; i++) {
          if (entries[i].isIntersecting) {
            youtube(entries[i].target);
            lazyobserver.unobserve(entries[i].target);
          }
        }
      },
      {
        threshold: [0.1],
        // Set a minimum delay between notifications
        delay: 100
      });
      // start observing
      for (var i=0; i < media.length; i++) {
        lazyobserver.observe(media[i]);
      }
    } else {
      // use eventListeners
      new Viewport(media, youtube);
    }
  }
})();
